#include "/settings.glsl"

#define UPSCALE_TERRAIN 1 //[0 1]
#define UPSCALE_ENTITIES 1 //[0 1]
#define UPSCALE_HAND_HELD 0 //[0 1]
#define UPSCALE_PARTICLES 0 //[0 1]

uniform sampler2D lightmap;

	#define COLORED_SHADOWS 1 //[0 1 2] //0: Stained glass will cast ordinary shadows. 1: Stained glass will cast colored shadows. 2: Stained glass will not cast any shadows. 
	#define SHADOW_BRIGHTNESS 0.75 //Light levels are multiplied by this number when the surface is in shadows [0.00 0.05 0.10 0.15 0.20 0.25 0.30 0.35 0.40 0.45 0.50 0.55 0.60 0.65 0.70 0.75 0.80 0.85 0.90 0.95 1.00]

	uniform sampler2D shadowcolor0;
	uniform sampler2D shadowtex0;
	uniform sampler2D shadowtex1;
	varying vec4 shadowPos;
#if SHADOWS == 1
#endif
uniform sampler2D texture;
uniform sampler2D normals;
uniform sampler2D specular;

uniform float far;
uniform float near;
uniform int isEyeInWater;
uniform float fogStart;
uniform float fogEnd;
uniform vec3 fogColor;
uniform float rainStrength;
uniform float sunAngle;

varying vec2 lmcoord;
varying vec2 texcoord;
varying vec4 glcolor;

varying vec4 viewPos;


//fix artifacts when colored shadows are enabled
const bool shadowcolor0Nearest = true;
const bool shadowtex0Nearest = true;
const bool shadowtex1Nearest = true;

//only using this include for shadowMapResolution,
//since that has to be declared in the fragment stage in order to do anything.
#include "/distort.glsl"




    vec2 dfdx = dFdx(texcoord.st);
	vec2 dfdy = dFdy(texcoord.st);
	varying  vec4 vlocal_uv_components;//CTMPOMFIX
	varying  vec4 vlocal_uv;//CTMPOMFIX
	#include "/timetravelbeards_Better_3D_Textures.glsl"




void main() {
	
	
	float dist = distance(vec3(0.),viewPos.xyz);
	#if TEXTURE_FILTERING_CPF > 0 && FILTER_HERE == 1
		vec4 color= atlas_uv_to_bilinear_data( texcoord,dist )*glcolor;
	#else
		vec4 color = texture2D(texture, texcoord) * glcolor;
		
	#endif
	
	float main_lighting = sunAngle <.5 ? SUN_BRIGHTNESS : MOON_BRIGHTNESS ;
	
	vec2 lm = lmcoord;
	
			//apply the color TORCH_BRIGHTNESS TORCH_FALLOFF SKY_LIGHT_BRIGHTNESS SKY_LIGHT_FALLOFF
						lm.x=pow(lm.x,TORCH_FALLOFF)*TORCH_BRIGHTNESS;
						lm.y=pow(lm.y,SKY_LIGHT_FALLOFF)*SKY_LIGHT_BRIGHTNESS;
						vec3 sky_color = 
						#if EXTRA_DARK_NIGHT > 0
							mix(
							texture2D(lightmap, vec2(1./32.,lm.y)).rgb,
							vec3(0.),
							clamp((1.-abs(sunAngle-.75)*4.)*10.,0.,1.)*float(EXTRA_DARK_NIGHT)*.1
							)
						#else
							texture2D(lightmap, vec2(1./32.,lm.y)).rgb
						#endif
						
						;
						#if CUSTOM_TORCH_COLOR == 0
							vec3 torch_color = 
								vec3(1.,0.9,0.8);
								//texture2D(lightmap, vec2(lm.x,0.1)).rgb;//
						#endif
						#if CUSTOM_TORCH_COLOR == 1
							vec3 torch_color = 
								vec3(TORCH_HI_R,TORCH_HI_G,TORCH_HI_B);
								
						#endif
						#if CUSTOM_TORCH_COLOR == 2
							vec3 torch_color = 
								mix(vec3(TORCH_LOW_R,TORCH_LOW_G,TORCH_LOW_B),
								vec3(TORCH_HI_R,TORCH_HI_G,TORCH_HI_B),
								lm.x)
								;
								
						#endif
						
						vec4 shadowLightColor=vec4(1.0);
	#if SHADOWS == 1
		
		if (shadowPos.w > 0.0) {
			//surface is facing towards shadowLightPosition
			#if COLORED_SHADOWS == 0
				//for normal shadows, only consider the closest thing to the sun,
				//regardless of whether or not it's opaque.
				if (texture2D(shadowtex0, shadowPos.xy).r < shadowPos.z) {
			#else
				//for invisible and colored shadows, first check the closest OPAQUE thing to the sun.
				if (texture2D(shadowtex1, shadowPos.xy).r < shadowPos.z) {
			#endif
				//surface is in shadows. reduce light level.
				//lm.y *= SHADOW_BRIGHTNESS;
				main_lighting = 0.0;
			}
			else {
				//surface is in direct sunlight. increase light level.
				//lm.y = mix(31.0 / 32.0 * SHADOW_BRIGHTNESS, 31.0 / 32.0, sqrt(shadowPos.w));
				#if COLORED_SHADOWS == 1
					//when colored shadows are enabled and there's nothing OPAQUE between us and the sun,
					//perform a 2nd check to see if there's anything translucent between us and the sun.
					if (texture2D(shadowtex0, shadowPos.xy).r < shadowPos.z) {
						//surface has translucent object between it and the sun. modify its color.
						//if the block light is high, modify the color less.
						 shadowLightColor = texture2D(shadowcolor0, shadowPos.xy);
						//make colors more intense when the shadow light color is more opaque.
						shadowLightColor.rgb = mix(vec3(1.0), shadowLightColor.rgb, shadowLightColor.a);
						//also make colors less intense when the block light level is high.
						shadowLightColor.rgb = mix(shadowLightColor.rgb, vec3(1.0), lm.x);
						
						
						
					}
				#endif
			}
		}


						
						color.rgb *= max(vec3(MINIMUM_LIGHT_LEVEL),
						 shadowPos.w*shadowLightColor.rgb*main_lighting
						*sky_color
						+lm.y*sky_color
						+lm.x*torch_color
						)
						;
	#else
	color.rgb *= 	max(vec3(MINIMUM_LIGHT_LEVEL),
						#if DIRECTIONAL_LIGHTING == 1
						 shadowPos.w*main_lighting
						#else 
						main_lighting
						
						#endif
						*sky_color
						+lm.y*sky_color
						+lm.x*torch_color
						);
						;
	
	#endif
	
	#if FOG == 1
		float water_fog = isEyeInWater == 1? 30. : isEyeInWater == 2? 20. : isEyeInWater == 1? 10. : 0.;
		water_fog = (water_fog > 1. ) ? clamp((dist)/water_fog,0.,1.) : 0.;
		//fogColor FOG_START FOG_END
			float border_fog_amount = clamp((dist-(BORDER_FOG_START*far))/(((1.-BORDER_FOG_START)*far)),0.,1.);
			float fog_amount = 
				clamp(
				max(
				water_fog,
				max( 
					clamp((dist-FOG_START)/(FOG_END-FOG_START),0.,FOG_MAX),
					border_fog_amount)
					*(1.+rainStrength)
					)
				,0.,1.)
				;
			color.rgb = mix(color.rgb,fogColor.rgb,fog_amount);
	#endif
	
	#if DONT_BLOW_OUT_WHITES == 1
		color.r = color.r <.95?color.r : .95+(color.r-.95)*.1;
		color.g = color.g <.95?color.g : .95+(color.g-.95)*.1;
		color.b = color.b <.95?color.b : .95+(color.b-.95)*.1;
	#endif

/* DRAWBUFFERS:0 */
	gl_FragData[0] = color; //gcolor
}